package edu.neu.daj;

import org.eclipse.ui.plugin.*;
import org.eclipse.core.resources.*;

import java.util.List;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Hashtable;
import java.io.File;

import org.eclipse.jdt.core.*;

import org.eclipse.ui.*;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.*;

//import org.eclipse.ajdt.internal.ui.views.*;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.widgets.Display;

/**
 * The main plugin class to be used in the desktop.
 */
public class DajPlugin extends AbstractUIPlugin {
	//The shared instance.
	
	public static final String VERSION = "0.1.0";
		public static final String PLUGIN_ID = "edu.neu.daj";
		public static final String ID_BUILDER = PLUGIN_ID + ".dajbuilder";
		public static final String ID_OUTLINE = PLUGIN_ID + ".dajoutlineview";
		public static final String ID_NATURE = PLUGIN_ID + ".dajnature";

		/**
		 * General debug trace for the plug-in enabled through the master trace
		 * switch.
		 */
		public static boolean isDebugging = false;

		/**
		 * More detailed trace for the builder. Controlled by options flag
		 * org.eclipse.ajdt.ui/builderDebug
		 */
		public static boolean DEBUG_BUILDER = false;

		/**
		 * More detailed trace for the compiler monitor. Controlled by options flag
		 * org.eclipse.ajdt.ui/builderDebug
		 */
		public static boolean DEBUG_COMPILER = false;

		/**
		 * More detailed trace for the outline view. Controlled by options flag
		 * org.eclipse.ajdt.ui/outlineDebug
		 */
		public static boolean DEBUG_OUTLINE = false;

		/**
		 * shared single instance of the plugin
		 */
		private static DajPlugin plugin;

		/**
		 * remembers the current build configuration file for each project
		 */
		private static Hashtable projectToBuildChoice = new Hashtable();

		/**
		 * plugin resource bundle used for NLS
		 */
		private ResourceBundle resourceBundle;

		/**
		 * The currently selected project
		 */
		private IProject currentProject;

		/**
		 * The currently selected resource 
		 */
		private IResource currentResource;

		/**
		 * The workbench Display for use by asynchronous UI updates
		 */
		private Display display;
	
		
		/**
		 * A resource change listener that will listen for new resource additions.
		 */
		IResourceChangeListener resourceChangeListener;
	
		private final static String defaultLstShouldBeUsed=
		  "edu.neu.daj.buildConfig.useDefaultLst";


		/**
		 * Set the build configuration file to be used when building this project
		 */
		public static void setBuildConfigurationFile(
			IProject project,
			IFile buildfile) {
			
			// Preserve the build selection choice in the preference store, with a
			// name unique to this project.
			IPreferenceStore store = DajPlugin.getDefault().getPreferenceStore();
	    
			String propertyName = "edu.neu.daj."+project.getName()+".lst";
	    
			if (buildfile== null)
			  store.setValue(propertyName, defaultLstShouldBeUsed);
			else 
			  store.setValue(propertyName,buildfile.getLocation().toOSString());
	      
	
			//String cfg = getBuildConfigurationFile( project );      
		}

		public static void setBuildConfigurationFile(
			IProject project,
			IResource buildfile) {
			
			if (buildfile == null) 
			  setBuildConfigurationFile(project,(IFile)null);
			else if (buildfile instanceof IFile) {
			  setBuildConfigurationFile(project,(IFile)buildfile);
			}	
		}
	

	/**
	 * Get the build configuration file to be used for building this project.
	 * If the user hasn't selected one, use "default.lst" in the project directory
	 */
	public static String getBuildConfigurationFile(IProject project) {
	
		IPreferenceStore store = DajPlugin.getDefault().getPreferenceStore();
 
		String propertyName = "edu.neu.daj."+project.getName()+".lst";
    
		String configFile = store.getString(propertyName);
	
		if (configFile == null || configFile.length()==0 || configFile.equals(defaultLstShouldBeUsed)) {
			configFile = project.getLocation().toOSString() + 
						 File.separator +
						 edu.neu.daj.internal.builder.Builder.DEFAULT_CONFIG_FILE;
			store.setValue( propertyName, configFile );
		} 
		if (isDebugging) {
			System.out.println("using: " + configFile);
		}
		return configFile;
	}

		// At startup, remember the display for later use and register to receive
		// information on part selection
		public void startup() throws CoreException {
			super.startup();
			display = Display.getCurrent();

//	  System.err.println("AspectJPlugin.startup: Adding selection listener...");
			if (plugin != null) {
//			System.err.println("Adding to selection service on workbench window: "+plugin.getWorkbench().getActiveWorkbenchWindow().toString());
				plugin
					.getWorkbench()
					.getActiveWorkbenchWindow()
					.getSelectionService();
			} else
				System.err.println("plugin is null");
		}

		/**
		 * get the active window in the workbench, or null if no window is active
		 */
		public org.eclipse.ui.IWorkbenchWindow getActiveWorkbenchWindow() {
			return plugin.getWorkbench().getActiveWorkbenchWindow();

		}

		/**
		 * get the main workbench display
		 */
		public Display getDisplay() {
			return display;
		}

		/**
		 * get the current project, if nobody has set a project yet, use the
		 * first project in the workspace
		 */
		public IProject getCurrentProject() {
			IProject current;
			if (currentProject != null) {
				current = currentProject;
			} else {
				IProject[] projects = getWorkspace().getRoot().getProjects();
				if (projects.length > 0) {
					current = projects[0];
				} else {
					current = null;
				}
			}
			return current;
		}

		/**
		 * get the current resource.  This method can return null if a resource
		 * has not been selected or the resource selected has no individual physical
		 * representation in the workspace.  For example, selecting an 'external'
		 * jar file within a project will cause currentProject to be set appropriately
		 * but there is no real resource representing that jar (its an artefact from
		 * outside the workbench) so currentResource will be null. 
		 */
		public IResource getCurrentResource() {
			return currentResource;
		}

		/**
		 * set the current project - called by the builder when we're about to
		 * do a build.
		 */
		public void setCurrentProject(IProject project) {
			currentProject = project;
		}

		/**
		 * set the current project by name
		 */
		public void setCurrentProject(String projectName) {
			boolean matched = false;
			IProject[] projects = getWorkspace().getRoot().getProjects();
			for (int i = 0; i < projects.length; i++) {
				if (projects[i].getName().equals(projectName)) {
					currentProject = projects[i];
					matched = true;
					break;
				}
			}
			if ( !matched ) { 
				/*getErrorHandler().handleWarning( 
					getResourceString( "bad.project" ) + " " + projectName );*/	
			}
		}

		/**
		 * initialize the plug-in image registry
		 */
		protected void initializeImageRegistry(ImageRegistry reg) {
		}

		/**
		 * Initialize plug-in debugging
		 */
		private void initDebugging() {
			if (isDebugging()) {
				System.out.println("DAJ START: " + PLUGIN_ID + " " + VERSION);
				isDebugging = true;

				String option;
				option = Platform.getDebugOption(PLUGIN_ID + "/builderDebug");
				if (option != null && option.equals("true")) {
					System.out.println("DAJ builderDebug ON");
					DEBUG_BUILDER = true;
				} else {
					System.out.println("DAJ builderDebug OFF");
				}
				option = Platform.getDebugOption(PLUGIN_ID + "/compilerDebug");
				if (option != null && option.equals("true")) {
					System.out.println("DAJ compilerDebug ON");
					DEBUG_COMPILER = true;
				} else {
					System.out.println("DAJ compilerDebug OFF");
				}
				option = Platform.getDebugOption(PLUGIN_ID + "/outlineDebug");
				if (option != null && option.equals("true")) {
					System.out.println("DAJ outlineDebug ON");
					DEBUG_OUTLINE = true;
				} else {
					System.out.println("DAJ outlineDebug OFF");
				}			
			}

		}

		// Implementation of ISelectionListener follows

		/** 
		 * Keeps the currentResource and currentProject information up to date
		 * in this class, as this method is called whenever a user changes
		 * their selection in the workspace.
		 */
		public void selectionChanged(IWorkbenchPart iwp, ISelection is) {
			try {
				// If we want to check only for selection changes in the Packages
				// view, then we could check the WorkbenchPart:
				//			if (iwp.getTitle().equals("Packages")) {
				// But there are so many places where the resources are exposed
				// navigator view, etc - that if we can be more generic and cope
				// with selection of the resources occurring anywhere, then we should always
				// have the current project correct.
				//System.out.println( "Selection changed: " +
				//	iwp.getTitle() + " " + is.toString() );
				
				// AMC note: GM1 build is firing an ITextSelection event only
				// clicking on the tab for an open file in the editor (to change
				// the current file being viewed). This does *not* give us the 
				// information we need to update the current project :-(

				if (is instanceof IStructuredSelection) {
					IStructuredSelection structuredSelection = (IStructuredSelection) is;
					Object o = structuredSelection.getFirstElement();

					if (o != null) {
						if (o instanceof IResource) {
							currentResource = (IResource) o;
							currentProject = currentResource.getProject();

						} else if (o instanceof IJavaElement) {
							IJavaElement je = (IJavaElement) o;
							if (je.getJavaProject() != null) {
								currentResource = je.getUnderlyingResource(); // Might be null!
								currentProject = je.getJavaProject().getProject();
							}

						//} else {
						//	System.err.println("Unrecognized selection is a ]" + o.toString() + "[");
						}
					}
				} else if ( is instanceof ITextSelection ) {
//					ITextSelection ts = (ITextSelection) is;
//					System.out.println( "Selected: " + ts.getText() );
				}
			
			
			} catch (Exception jme) {
				/*getErrorHandler().handleError(
					"Exception in AspectJPlugin.selectionChanged processing",
					jme);*/
			}

		}
	
	
		/**
		 * Build a list of .lst files in the currently selected project - EXCEPT the
		 * default.lst file.  It uses the helper method getLstFiles() defined 
		 * below to perform recursion.
		 * 
		 * @return List of IResource objects that represent .lst files in the current project 
		 */
		public List getListOfConfigFilesForCurrentProject() {
			if (currentProject == null ) return null;
			List allLstFiles = new ArrayList();
			try {
				IResource[] files = currentProject.members();
				getLstFiles(files, allLstFiles);
			} catch (CoreException ce) {
				/*DajPlugin.getDefault().getErrorHandler().handleError(
				"Exception occurred whilst retrieving all .lst files in a project",ce);*/
			}
			return allLstFiles;
		}

		/**
		 * Find all the ".lst" files in the project.  Populates the List parameter
		 * passed in using recursion to traverse the whole resource hierarchy for
		 * the project.
		 */
		private void getLstFiles(IResource[] resource_list, List allLstFiles) {
			try {
				for (int i = 0; i < resource_list.length; i++) {
					IResource ir = resource_list[i];
					// Add lst files to the list, but NOT default.lst
					if (ir.getName().endsWith(".lst") && !ir.getName().equals("default.lst"))
						allLstFiles.add(ir);
					if (ir instanceof IContainer)
						getLstFiles(((IContainer) ir).members(), allLstFiles);
				}
			} catch (CoreException ce) {
				/*DajPlugin.getDefault().getErrorHandler().handleError(
				"Exception occurred whilst retrieiving all .lst files in a project",ce);*/
			}
		}
	
	/**
	 * The constructor.
	 */
	public DajPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle("edu.neu.daj.DajPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * Returns the shared instance.
	 */
	public static DajPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= DajPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
}
