package edu.neu.daj.internal.builder;

import java.util.*;
import java.io.*;
import edu.neu.daj.DajPlugin;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.preference.IPreferenceStore;

public class Builder extends IncrementalProjectBuilder {

	/**
	 * The name of the default build config file for an DAJ project
	 */
	public static final String DEFAULT_CONFIG_FILE = "default.lst";
	
	public static final int CONFIG_FILE_WRITE_ERROR = -10;

	/**
	 * The last project we did a build for, needed by content outline
	 * view to decide which updates to accept.
	 */
	private static IProject lastBuiltProject = null;

	/**
	 * Constructor for DajBuilder
	 */
	public Builder() {}


	public static IProject getLastBuildTarget( ) {
		return lastBuiltProject;
	}

	protected IProject[] build(
		int kind,
		Map args,
		IProgressMonitor progressMonitor)
		throws CoreException {
		IPreferenceStore store = DajPlugin.getDefault().getPreferenceStore();		
		try {

			DajPlugin dajPlugin = DajPlugin.getDefault();
			String cpath = store.getString("dajPathPreference") + ":"
					+ store.getString("aspectjPathPreference") + ":"
					+ store.getString("aspectjToolsPathPreference") + ":"
					+ store.getString("antlrPathPreference") + ":"
					+ store.getString("demeterjPathPreference");

			IProject project = getProject();
			dajPlugin.setCurrentProject(project);
			
			Runtime runtime = Runtime.getRuntime();
			String path =   DajPlugin.getWorkspace().getRoot().getLocation().toString() +
							dajPlugin.getCurrentProject().getFullPath().toString();
			
			// Bug: DAJ can't access the files
			
			/*File tempDir = new File(path + ".." + File.separator + "DAJtmp000ij");
			tempDir.mkdir();
			
			String tmpPath = tempDir.getCanonicalPath();
			
			File f = new File(path);
			File[] fl = f.listFiles();
			File fout;
			
			for(int z = 0; z < fl.length; z++) {
				if(fl[z].isFile()) {
					fout = new File(tempDir + File.separator + fl[z].getName());
					copyFile(fl[z], fout);
				}
			}*/
			
			Process p = runtime.exec(
				"java -cp $CLASSPATH:" + cpath + 
				" edu.neu.ccs.demeter.tools.daj.DAJ -o "
				+ path + File.separator + "gen -d "
				+ path + File.separator + "classes "
				+ path + File.separator + "*.java "
				+ path + File.separator + "*.cd "
				+ path + File.separator + "*.trv");
				
				
			int counter = 0;
			BufferedReader i = new BufferedReader(new InputStreamReader(p.getInputStream()));
			
			while(counter < 100) {
				if (i.readLine() != null) {
					return null;
				}
				counter++;
			}	
				
			lastBuiltProject = project;
            
		} catch (Throwable t) {
			System.out.println("DAJ encountered errors." + t.toString());
		}

		IProject[] requiredResourceDeltasOnNextInvocation = null;
		return requiredResourceDeltasOnNextInvocation;
	}
	
	public void copyFile(File in, File out) throws Exception {
		FileInputStream fis = new FileInputStream(in);
		FileOutputStream fos = new FileOutputStream(out);
		byte[] buf = new byte[1024];
		int i = 0;
		while((i=fis.read(buf)) !=-1) {
			fos.write(buf, 0, i);
		}
		fis.close();
		fos.close();
	}

	/**
	 * Get the fully qualified path to the build configuation file specified
	 * for this project.
	 */
	private String getBuildFilePath(IProject project) {
		String buildfile = DajPlugin.getBuildConfigurationFile(project);
		return buildfile;
	}

	/**
	 * If the project uses a default build file, then it will be regenerated
	 * to ensure that all project files are included.
	 */
	private void updateBuildConfigIfNecessary(IProject project, List projectFiles)
		throws CoreException {
		if (getBuildFilePath(project).endsWith(DEFAULT_CONFIG_FILE)) {
			writeBuildConfigFile(projectFiles, project);
		}
	}

	/**
	 * Create a full build configuration file for this project
	 */
	private void writeBuildConfigFile(List projectFiles, IProject project)
		throws CoreException {
		String configurationFilename = getBuildFilePath(project);

		try {
			FileWriter fw = new FileWriter(configurationFilename);
			BufferedWriter bw = new BufferedWriter(fw);
			for (Iterator it = projectFiles.iterator(); it.hasNext();) {
				File jf = (File) it.next();
				String fileName = jf.toString();
				if (fileName.endsWith(".java") || fileName.endsWith(".aj")) {
					bw.write(fileName);
					bw.write("\n");
				}
			}
			bw.flush();
			fw.flush();
			fw.close();
		} catch (Exception e) {
			Status status =
				new Status(
					Status.ERROR,
					DajPlugin.PLUGIN_ID,
					CONFIG_FILE_WRITE_ERROR,
					DajPlugin.getResourceString("configFileCreateError"),
					e);
			throw new CoreException(status);
		}

	}
}
