package edu.neu.daj.internal.ui;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IWorkbench;
import org.eclipse.jdt.core.JavaCore;

import edu.neu.daj.DajPlugin;

/**
 * @author nick
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class DajProjectNature implements IProjectNature {

	private IProject project;

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IProjectNature#configure()
	 */
	public void configure() throws CoreException {

		IProjectDescription projectDescription = project.getDescription();
		ICommand command = projectDescription.newCommand();
		command.setBuilderName(DajPlugin.ID_BUILDER);

		ICommand[] buildCommands = projectDescription.getBuildSpec();
		ICommand[] newBuildCommands;
		if (contains(buildCommands, JavaCore.BUILDER_ID)) {
			newBuildCommands =
				swap(buildCommands, JavaCore.BUILDER_ID, command );
		} else {
			newBuildCommands = insert(buildCommands, command);
		}
		projectDescription.setBuildSpec(newBuildCommands);
			project.setDescription(projectDescription, null);
		
		IWorkbench workbench = DajPlugin.getDefault().getWorkbench();
		workbench.showPerspective( "org.eclipse.jdt.ui.JavaPerspective" , workbench.getActiveWorkbenchWindow() );		
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IProjectNature#deconfigure()
	 */
	public void deconfigure() throws CoreException {
		IProjectDescription description = project.getDescription();
		ICommand[] buildCommands = description.getBuildSpec();
		ICommand command = description.newCommand();
		command.setBuilderName(JavaCore.BUILDER_ID);
		
		ICommand[] newBuildCommands;
		if ( contains( buildCommands, DajPlugin.ID_BUILDER ) ) {
			newBuildCommands = swap( buildCommands, DajPlugin.ID_BUILDER, command );
		} else {
			newBuildCommands = remove( buildCommands, DajPlugin.ID_BUILDER );
		}			
		
		description.setBuildSpec(newBuildCommands);
		project.setDescription(description, null);

	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IProjectNature#getProject()
	 */
	public IProject getProject() {
		return project;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.resources.IProjectNature#setProject(org.eclipse.core.resources.IProject)
	 */
	public void setProject(IProject project) {
		this.project = project;
	}
	
	/**
	 * Check if the given biuld command list contains a given command
	 */
	private boolean contains(ICommand[] commands, String builderId) {
		boolean found = false;
		for (int i = 0; i < commands.length; i++) {
			if (commands[i].getBuilderName().equals(builderId)) {
				found = true;
				break;
			}
		}
		return found;
	}

	/**
	 * In a list of build commands, swap all occurences of one entry for
	 * another
	 */
	private ICommand[] swap(
		ICommand[] sourceCommands,
		String oldBuilderId,
		ICommand newCommand) 
	{
		ICommand[] newCommands = new ICommand[sourceCommands.length];
		for ( int i = 0; i < sourceCommands.length; i++ ) {
			if ( sourceCommands[i].getBuilderName( ).equals( oldBuilderId ) ) {
				newCommands[i] = newCommand;
			} else {
				newCommands[i] = sourceCommands[i];
			}
		}	
		return newCommands;	
	}
	
	/**
	 * Insert a new build command at the front of an existing list
	 */
	private ICommand[] insert( ICommand[] sourceCommands, ICommand command ) {
		ICommand[] newCommands = new ICommand[ sourceCommands.length + 1 ];
		newCommands[0] = command;
		for (int i = 0; i < sourceCommands.length; i++ ) {
			newCommands[i+1] = sourceCommands[i];
		}		
		return newCommands;		
	}
	

	private ICommand[] remove( ICommand[] sourceCommands, String builderId ) {
		ICommand[] newCommands = new ICommand[ sourceCommands.length - 1 ];
		int newCommandIndex = 0;
		for (int i = 0; i < sourceCommands.length; i++ ) {
			if ( !sourceCommands[i].getBuilderName( ).equals( builderId ) ) {
				newCommands[newCommandIndex++] = sourceCommands[i];
			}
		}		
		return newCommands;		
	}

}
